@@ -2,7 +2,7 @@
 # encoding=UTF-8
 
 # Copyright © 2011 Jakub Wilk <jwilk@debian.org>
-#           © 2013-2017 Dmitry Shachnev <mitya57@debian.org>
+#           © 2013-2022 Dmitry Shachnev <mitya57@debian.org>
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -33,8 +33,8 @@ import re
 import unittest
 import gi
 
-gi.require_version('Gtk', '3.0')
-gi.require_version('WebKit2', '4.1')
+gi.require_version('Gtk', '4.0')
+gi.require_version('WebKit2', '5.0')
 from gi.repository import GLib, Gtk, WebKit2
 
 default_time_limit = 40.0
@@ -56,6 +56,13 @@ class Browser(object):
         self._view.connect('notify::title', self._on_title_changed)
         self._result = None
         self._id = 0
+        self._application = Gtk.Application()
+        self._application.connect('activate', self._activate)
+
+    def _activate(self, application):
+        self._window = Gtk.ApplicationWindow(application=application)
+        self._window.set_child(self._view)
+        self._window.show()
 
     def _on_title_changed(self, webview, user_data):
         contents = webview.get_property('title')
@@ -63,7 +70,7 @@ class Browser(object):
         found = re.match(r"(?P<n_results>\d+) (?P<n_links>\d+) (?P<n_highlights>\d+)", contents)
         if found:
             self._result = found.groupdict()
-            Gtk.main_quit()
+            self._window.destroy()
             GLib.source_remove(self._id)
             self._id = 0
 
@@ -75,7 +82,7 @@ class Browser(object):
             "document.title = `${n_results} ${n_links} ${n_highlights}`;")
         if self._time_limit < 0:
             self._result = None
-            Gtk.main_quit()
+            self._window.destroy()
             return GLib.SOURCE_REMOVE
 
         self._time_limit -= 1
@@ -85,7 +92,7 @@ class Browser(object):
         self._view.load_uri(url)
         self._time_limit = time_limit
         self._id = GLib.timeout_add_seconds(1, self._quit)
-        Gtk.main()
+        self._application.run()
         if self._result is None:
             raise Timeout
         return self._result
